/**
 * @module @nativescript-community/ui-material-core-tabs/tab-navigation-base
 */
import { AddArrayFromBuilder, AddChildFromBuilder, CoercibleProperty, Color, EventData, Property, View, ViewBase } from '@nativescript/core';
import { TabNavigationBase as TabNavigationBaseDefinition } from '.';
import { TabContentItem } from '../tab-content-item';
import { TabStrip } from '../tab-strip';
import { TabStripItem } from '../tab-strip-item';
/**
 * Defines the data for the tab navigation selectedIndexChanged event.
 */
export interface SelectedIndexChangedEventData extends EventData {
    /**
     * The old selected index.
     */
    oldIndex: number;
    /**
     * The new selected index.
     */
    newIndex: number;
}
export declare class TabNavigationBase extends View implements TabNavigationBaseDefinition, AddChildFromBuilder, AddArrayFromBuilder {
    static selectedIndexChangedEvent: string;
    static selectedEvent: string;
    static unselectedEvent: string;
    unloadOnTabChange: boolean;
    items: TabContentItem[];
    tabStrip: TabStrip;
    selectedIndex: number;
    _addArrayFromBuilder(name: string, value: any[]): void;
    _addChildFromBuilder(name: string, value: any): void;
    get _selectedView(): View;
    get _childrenCount(): number;
    eachChild(callback: (child: ViewBase) => boolean): void;
    eachChildView(callback: (child: View) => boolean): void;
    onItemsChanged(oldItems: TabContentItem[], newItems: TabContentItem[]): void;
    onTabStripChanged(oldTabStrip: TabStrip, newTabStrip: TabStrip): void;
    onSelectedIndexChanged(oldIndex: number, newIndex: number): void;
    getTabBarBackgroundColor(): any;
    getTabBarBackgroundArgbColor(): any;
    setTabBarBackgroundColor(value: any): void;
    getTabBarFontInternal(): any;
    setTabBarFontInternal(value: any): void;
    getTabBarTextTransform(): any;
    setTabBarTextTransform(value: any): void;
    getTabBarHighlightColor(): any;
    setTabBarHighlightColor(value: any): void;
    getTabBarSelectedItemColor(): Color;
    setTabBarSelectedItemColor(value: Color): void;
    getTabBarUnSelectedItemColor(): Color;
    setTabBarUnSelectedItemColor(value: Color): void;
    getTabBarColor(): any;
    setTabBarColor(value: any): void;
    setTabBarItemTitle(tabStripItem: TabStripItem, value: any): void;
    getTabBarItemBackgroundColor(tabStripItem: TabStripItem): any;
    setTabBarItemBackgroundColor(tabStripItem: TabStripItem, value: any): void;
    getTabBarItemColor(tabStripItem: TabStripItem): any;
    setTabBarItemColor(tabStripItem: TabStripItem, value: any): void;
    setTabBarIconColor(tabStripItem: TabStripItem, value: any): void;
    setTabBarIconSource(tabStripItem: TabStripItem, value: any): void;
    getTabBarItemFontSize(tabStripItem: TabStripItem): any;
    setTabBarItemFontSize(tabStripItem: TabStripItem, value: any): void;
    getTabBarItemFontInternal(tabStripItem: TabStripItem): any;
    setTabBarItemFontInternal(tabStripItem: TabStripItem, value: any): void;
    getTabBarItemTextTransform(tabStripItem: TabStripItem): any;
    setTabBarItemTextTransform(tabStripItem: TabStripItem, value: any): void;
    setTabBarRippleColor(value: Color, alpha?: number): void;
    getTabBarRippleColor(): any;
    /**
     * Gets the parent fragment manager from a fragment to be used in destroying or hiding it.
     * @param fragment target fragment
     * @returns the parent fragment manager or null if none exists.
     */
    _getParentFragmentManagerFromFragment(fragment: androidx.fragment.app.Fragment): androidx.fragment.app.FragmentManager;
}
export declare function getIconSpecSize(size: {
    width: number;
    height: number;
}): {
    width: number;
    height: number;
};
export declare const selectedIndexProperty: CoercibleProperty<TabNavigationBaseDefinition, number>;
export declare const _tabs: WeakRef<TabNavigationBaseDefinition>[];
export declare const itemsProperty: Property<TabNavigationBaseDefinition, TabContentItem[]>;
export declare const tabStripProperty: Property<TabNavigationBaseDefinition, TabStrip>;
export declare const unloadOnTabChangeProperty: Property<TabNavigationBaseDefinition, boolean>;
