import { cssProperty } from '@nativescript-community/ui-material-core';
// Types
import { Frame, Property, booleanConverter } from '@nativescript/core';
import { TabNavigationBase as TabNavigationBaseBase } from '../tab-navigation-base';
export var knownCollections;
(function (knownCollections) {
    knownCollections.items = 'items';
})(knownCollections || (knownCollections = {}));
export var TabsPosition;
(function (TabsPosition) {
    TabsPosition["Top"] = "top";
    TabsPosition["Bottom"] = "bottom";
})(TabsPosition || (TabsPosition = {}));
export class TabNavigationBase extends TabNavigationBaseBase {
    constructor() {
        super(...arguments);
        this.mTextTransform = 'none';
    }
    getItemLabelTextTransform(tabStripItem) {
        const nestedLabel = tabStripItem.label;
        let textTransform = null;
        if (nestedLabel && nestedLabel.style.textTransform !== 'initial') {
            textTransform = nestedLabel.style.textTransform;
        }
        else if (tabStripItem.style.textTransform !== 'initial') {
            textTransform = tabStripItem.style.textTransform;
        }
        return textTransform || this.mTextTransform;
    }
    getTabBarTextTransform() {
        return this.mTextTransform;
    }
    getTabBarItemTextTransform(tabStripItem) {
        return this.getItemLabelTextTransform(tabStripItem);
    }
    _loadUnloadTabItems(newIndex) {
        const items = this.items;
        if (!items) {
            return;
        }
        // const offsideItems = this.offscreenTabLimit;
        // use 0 because users expect load/unload to be like selected/unselected
        const offsideItems = 0;
        const toLoad = [];
        const toUnload = [];
        for (let index = 0; index < items.length; index++) {
            const item = items[index];
            if (item) {
                if (Math.abs(index - newIndex) <= offsideItems) {
                    toLoad.push(item);
                }
                else {
                    toUnload.push(item);
                }
            }
        }
        if (this.unloadOnTabChange) {
            toUnload.forEach((item) => item.unloadView(item.content));
        }
        const newItem = items[newIndex];
        const selectedView = newItem && newItem.content;
        if (selectedView instanceof Frame) {
            selectedView._pushInFrameStackRecursive();
        }
        if (this.isLoaded) {
            toLoad.forEach((item) => item.loadView(item.content));
        }
    }
}
__decorate([
    cssProperty
], TabNavigationBase.prototype, "rippleColor", void 0);
export const swipeEnabledProperty = new Property({
    name: 'swipeEnabled',
    defaultValue: true,
    valueConverter: booleanConverter
});
swipeEnabledProperty.register(TabNavigationBase);
export const offscreenTabLimitProperty = new Property({
    name: 'offscreenTabLimit',
    defaultValue: 1,
    valueConverter: (v) => parseInt(v, 10)
});
offscreenTabLimitProperty.register(TabNavigationBase);
export const tabsPositionProperty = new Property({ name: 'tabsPosition', defaultValue: TabsPosition.Top });
tabsPositionProperty.register(TabNavigationBase);
// export type IOSTabBarItemsAlignment = 'leading' | 'justified' | 'center' | 'centerSelected';
// export const iOSTabBarItemsAlignmentProperty = new Property<TabsBase, IOSTabBarItemsAlignment>({ name: 'iOSTabBarItemsAlignment', defaultValue: 'justified' });
// iOSTabBarItemsAlignmentProperty.register(TabsBase);
export const animationEnabledProperty = new Property({ name: 'animationEnabled', defaultValue: true, valueConverter: booleanConverter });
animationEnabledProperty.register(TabNavigationBase);
//# sourceMappingURL=index-common.js.map