import { Application, Color, ImageSource, Utils, getTransformedText } from '@nativescript/core';
import { TabContentItem } from '../tab-content-item';
import { getIconSpecSize, itemsProperty, selectedIndexProperty, tabStripProperty } from '../tab-navigation-base';
import { TabStrip } from '../tab-strip';
import { TabStripItem } from '../tab-strip-item';
import { TabNavigationBase, TabsPosition, offscreenTabLimitProperty, swipeEnabledProperty } from './index-common';
export * from './index-common';
export { TabContentItem, TabStrip, TabStripItem };
const ACCENT_COLOR = 'colorAccent';
export const PRIMARY_COLOR = 'colorPrimary';
const DEFAULT_ELEVATION = 4;
const TABID = '_tabId';
const INDEX = '_index';
class IconInfo {
}
// eslint-disable-next-line no-redeclare
let PagerAdapter;
// eslint-disable-next-line no-redeclare
let PageChangeCallback;
let appResources;
function getTabById(id) {
    const ref = tabs.find((ref) => {
        const tab = ref.get();
        return tab && tab._domId === id;
    });
    return ref && ref.get();
}
function initializeNativeClasses() {
    if (PagerAdapter) {
        return;
    }
    var PageChangeCallbackImpl = /** @class */ (function (_super) {
    __extends(PageChangeCallbackImpl, _super);
    function PageChangeCallbackImpl(owner) {
        var _this = _super.call(this) || this;
        _this.owner = owner;
        return global.__native(_this);
    }
    PageChangeCallbackImpl.prototype.onPageSelected = function (position) {
        var _a;
        var owner = (_a = this.owner) === null || _a === void 0 ? void 0 : _a.get();
        if (owner) {
            owner.selectedIndex = position;
            var tabItems = owner.items;
            var newTabItem = tabItems ? tabItems[position] : null;
            if (newTabItem) {
                owner._loadUnloadTabItems(owner.selectedIndex);
            }
        }
    };
    return PageChangeCallbackImpl;
}(androidx.viewpager2.widget.ViewPager2.OnPageChangeCallback));
    var TabFragmentImplementation = /** @class */ (function (_super) {
    __extends(TabFragmentImplementation, _super);
    // private backgroundBitmap: android.graphics.Bitmap = null;
    function TabFragmentImplementation() {
        var _this = _super.call(this) || this;
        return global.__native(_this);
    }
    TabFragmentImplementation.newInstance = function (tabId, index) {
        var args = new android.os.Bundle();
        args.putInt(TABID, tabId);
        args.putInt(INDEX, index);
        var fragment = new TabFragmentImplementation();
        fragment.setArguments(args);
        return fragment;
    };
    TabFragmentImplementation.prototype.onCreate = function (savedInstanceState) {
        _super.prototype.onCreate.call(this, savedInstanceState);
        var args = this.getArguments();
        this.owner = getTabById(args.getInt(TABID));
        this.index = args.getInt(INDEX);
        if (!this.owner) {
            throw new Error('Cannot find TabView');
        }
    };
    TabFragmentImplementation.prototype.onCreateView = function (inflater, container, savedInstanceState) {
        var tabItem = this.owner.items[this.index];
        tabItem.canBeLoaded = true;
        // ensure item is loaded
        tabItem.loadView(tabItem.content);
        return tabItem.nativeViewProtected;
    };
    TabFragmentImplementation.prototype.onDestroyView = function () {
        _super.prototype.onDestroyView.call(this);
        var tabItem = this.owner.items[this.index];
        tabItem.canBeLoaded = false;
        tabItem.unloadView(tabItem.content);
    };
    return TabFragmentImplementation;
}(org.nativescript.widgets.FragmentBase));
    var FragmentPagerAdapter = /** @class */ (function (_super) {
    __extends(FragmentPagerAdapter, _super);
    function FragmentPagerAdapter(owner, fragmentActivity) {
        var _this = _super.call(this, fragmentActivity) || this;
        _this.owner = owner;
        return global.__native(_this);
    }
    FragmentPagerAdapter.prototype.getItemCount = function () {
        var _a;
        var owner = (_a = this.owner) === null || _a === void 0 ? void 0 : _a.get();
        if (!owner) {
            return 0;
        }
        return owner.items.length;
    };
    FragmentPagerAdapter.prototype.createFragment = function (position) {
        var _a;
        var owner = (_a = this.owner) === null || _a === void 0 ? void 0 : _a.get();
        if (!owner) {
            return null;
        }
        var fragment = TabFragmentImplementation.newInstance(owner._domId, position);
        return fragment;
    };
    return FragmentPagerAdapter;
}(androidx.viewpager2.adapter.FragmentStateAdapter));
    PagerAdapter = FragmentPagerAdapter;
    PageChangeCallback = PageChangeCallbackImpl;
    appResources = Application.android.context.getResources();
}
let defaultAccentColor;
export function getDefaultAccentColor(context) {
    if (defaultAccentColor === undefined) {
        //Fallback color: https://developer.android.com/samples/SlidingTabsColors/src/com.example.android.common/view/SlidingTabStrip.html
        defaultAccentColor = Utils.android.resources.getPaletteColor(ACCENT_COLOR, context) || 0xff33b5e5;
    }
    return defaultAccentColor;
}
function setElevation(grid, tabsBar, tabsPosition) {
    const compat = androidx.core.view.ViewCompat;
    if (compat.setElevation) {
        const val = DEFAULT_ELEVATION * Utils.layout.getDisplayDensity();
        if (grid && tabsPosition === 'top') {
            compat.setElevation(grid, val);
        }
        if (tabsBar) {
            compat.setElevation(tabsBar, val);
        }
    }
}
export const tabs = new Array();
//TODO: move to abstract
export class TabNavigation extends TabNavigationBase {
    constructor() {
        super();
        this.mAndroidViewId = -1;
        this.mTextTransform = 'none';
        this.fragments = [];
        tabs.push(new WeakRef(this));
    }
    get _hasFragments() {
        return true;
    }
    onItemsChanged(oldItems, newItems) {
        super.onItemsChanged(oldItems, newItems);
        if (oldItems) {
            oldItems.forEach((item, i, arr) => {
                item.index = 0;
                item.tabItemSpec = null;
                item.setNativeView(null);
            });
        }
    }
    createNativeView() {
        initializeNativeClasses();
        const context = this._context;
        const nativeView = new org.nativescript.widgets.GridLayout(context);
        const viewPager = new androidx.viewpager2.widget.ViewPager2(context);
        viewPager.setSaveEnabled(false);
        const lp = new org.nativescript.widgets.CommonLayoutParams();
        lp.row = 1;
        if (this.tabsPosition === 'top') {
            nativeView.addRow(new org.nativescript.widgets.ItemSpec(1, org.nativescript.widgets.GridUnitType.auto));
            nativeView.addRow(new org.nativescript.widgets.ItemSpec(1, org.nativescript.widgets.GridUnitType.star));
            viewPager.setLayoutParams(lp);
        }
        else {
            nativeView.addRow(new org.nativescript.widgets.ItemSpec(1, org.nativescript.widgets.GridUnitType.star));
            nativeView.addRow(new org.nativescript.widgets.ItemSpec(1, org.nativescript.widgets.GridUnitType.auto));
            this.tabBarLayoutParams = lp;
        }
        nativeView.addView(viewPager);
        nativeView.viewPager = viewPager;
        const adapter = (this.mPagerAdapter = new PagerAdapter(new WeakRef(this), this._context));
        viewPager.setAdapter(adapter);
        viewPager.adapter = adapter;
        this.mViewPager = viewPager;
        setElevation(nativeView, null, this.tabsPosition);
        if (this.tabStrip) {
            this.handleTabStripChanged(nativeView, true, this.tabStrip);
        }
        return nativeView;
    }
    handleTabStripChanged(nativeView, isNewView, newTabStrip) {
        if (this.mTabsBar) {
            if (!isNewView)
                nativeView.removeView(this.mTabsBar);
            nativeView['tabsBar'] = null;
            this.mTabsBar = null;
        }
        if (newTabStrip) {
            initializeNativeClasses();
            const context = this._context;
            const tabsBar = (this.mTabsBar = this.createNativeTabBar(context));
            setElevation(null, tabsBar, this.tabsPosition);
            if (this.tabsPosition !== TabsPosition.Top) {
                tabsBar.setLayoutParams(this.tabBarLayoutParams);
            }
            nativeView.addView(tabsBar);
            nativeView['tabsBar'] = tabsBar;
            this.setTabStripItems(newTabStrip?.items || null);
        }
    }
    onTabStripChanged(oldTabStrip, newTabStrip) {
        super.onTabStripChanged(oldTabStrip, newTabStrip);
        const nativeView = this.nativeViewProtected;
        if (!nativeView) {
            return;
        }
        this.handleTabStripChanged(nativeView, false, newTabStrip);
    }
    onSelectedIndexChanged(oldIndex, newIndex) {
        if (this.mTabsBar) {
            this.selectTabBar(oldIndex, newIndex);
        }
        super.onSelectedIndexChanged(oldIndex, newIndex);
    }
    initNativeView() {
        super.initNativeView();
        if (this.mAndroidViewId < 0) {
            this.mAndroidViewId = android.view.View.generateViewId();
        }
        this.mPageListener = new PageChangeCallback(new WeakRef(this));
        this.mViewPager.setId(this.mAndroidViewId);
        this.mViewPager.registerOnPageChangeCallback(this.mPageListener);
    }
    disposeNativeView() {
        if (this.mTabsBar) {
            this.setTabBarItems(null);
        }
        // setAdapter(null) will destroy fragments
        this.mViewPager.setAdapter(null);
        this.mViewPager.unregisterOnPageChangeCallback(this.mPageListener);
        this.mPagerAdapter = null;
        super.disposeNativeView();
    }
    onLoaded() {
        super.onLoaded();
        // if (this._originalBackground) {
        //     console.log('setting original background', this._originalBackground);
        //     this.backgroundColor = null;
        //     this.backgroundColor = this._originalBackground;
        //     this._originalBackground = null;
        // }
        // this.setItems(this.items as any);
        if (this.tabStrip) {
            this.setTabStripItems(this.tabStrip.items);
        }
    }
    onUnloaded() {
        super.onUnloaded();
        // this.setItems(null);
        // this.setTabStripItems(null);
        // TODO: needed ?
        // this.items.forEach((item, i) => {
        //     item.unloadView(item.content);
        // });
    }
    shouldUpdateAdapter(items) {
        if (!this.mPagerAdapter) {
            return false;
        }
        const currentPagerAdapterItems = this.mPagerAdapter.items;
        // if both values are null, should not update
        if (!items && !currentPagerAdapterItems) {
            return false;
        }
        // if one value is null, should update
        if (!items || !currentPagerAdapterItems) {
            return true;
        }
        // if both are Arrays but length doesn't match, should update
        if (items.length !== currentPagerAdapterItems.length) {
            return true;
        }
        const matchingItems = currentPagerAdapterItems.filter((currentItem) => !!items.filter((item) => item._domId === currentItem._domId)[0]);
        // if both are Arrays and length matches, but not all items are the same, should update
        if (matchingItems.length !== items.length) {
            return true;
        }
        // if both are Arrays and length matches and all items are the same, should not update
        return false;
    }
    setItems(items) {
        if (items && items.length) {
            items.forEach((item, i) => {
                item.index = i;
            });
        }
        this._loadUnloadTabItems(this.selectedIndex);
        if (this.shouldUpdateAdapter(items)) {
            this.mPagerAdapter.notifyDataSetChanged();
        }
    }
    setTabStripItems(items) {
        const length = items ? items.length : 0;
        if (length === 0) {
            if (this.mTabsBar) {
                this.setTabBarItems(null);
            }
            return;
        }
        const tabItems = new Array();
        items.forEach((tabStripItem, i, arr) => {
            tabStripItem.index = i;
            const tabItemSpec = this.createTabItemSpec(tabStripItem);
            tabStripItem.tabItemSpec = tabItemSpec;
            tabItems.push(tabItemSpec);
        });
        const tabsBar = this.mTabsBar;
        this.setTabBarItems(tabItems);
        this.tabStrip.setNativeView(tabsBar);
        items.forEach((item, i, arr) => {
            const tv = this.getTabBarItemTextView(i);
            item.setNativeView(tv);
            this._setItemColor(item);
        });
    }
    createTabItemSpec(tabStripItem) {
        const tabItemSpec = new com.nativescript.material.core.TabItemSpec();
        if (tabStripItem.isLoaded) {
            const nestedLabel = tabStripItem.label;
            let title = nestedLabel.text;
            // TEXT-TRANSFORM
            const textTransform = this.getItemLabelTextTransform(tabStripItem);
            title = getTransformedText(title, textTransform);
            tabItemSpec.title = title;
            // BACKGROUND-COLOR
            const backgroundColor = tabStripItem.style.backgroundColor;
            tabItemSpec.backgroundColor = backgroundColor ? backgroundColor.android : this.getTabBarBackgroundArgbColor();
            // COLOR
            const itemColor = this.selectedIndex === tabStripItem.index ? this.mSelectedItemColor : this.mUnSelectedItemColor;
            const color = itemColor || nestedLabel.style.color;
            tabItemSpec.color = color && color.android;
            // FONT
            const fontInternal = nestedLabel.style.fontInternal;
            if (fontInternal) {
                tabItemSpec.fontSize = fontInternal.fontSize;
                tabItemSpec.typeFace = fontInternal.getAndroidTypeface();
            }
            // ICON
            const iconSource = tabStripItem.image && tabStripItem.image.src;
            if (iconSource) {
                const iconInfo = this.getIconInfo(tabStripItem, itemColor);
                if (iconInfo) {
                    // TODO: Make this native call that accepts string so that we don't load Bitmap in JS.
                    // tslint:disable-next-line:deprecation
                    tabItemSpec.iconDrawable = iconInfo.drawable;
                    tabItemSpec.imageHeight = iconInfo.height;
                }
                else {
                    // TODO:
                    // traceMissingIcon(iconSource);
                }
            }
        }
        return tabItemSpec;
    }
    getOriginalIcon(tabStripItem, color) {
        const iconSource = tabStripItem.image?.src;
        if (!iconSource) {
            return null;
        }
        let is;
        if (typeof iconSource === 'string') {
            if (Utils.isFontIconURI(iconSource)) {
                const fontIconCode = iconSource.split('//')[1];
                const target = tabStripItem.image ? tabStripItem.image : tabStripItem;
                const font = target.style.fontInternal;
                if (!color) {
                    color = target.style.color;
                }
                is = ImageSource.fromFontIconCodeSync(fontIconCode, font, color);
            }
            else {
                is = ImageSource.fromFileOrResourceSync(iconSource);
            }
        }
        else {
            is = iconSource;
        }
        return is?.android;
    }
    getDrawableInfo(image) {
        if (image) {
            if (this.tabStrip && this.tabStrip.isIconSizeFixed) {
                image = this.getFixedSizeIcon(image);
            }
            const imageDrawable = new android.graphics.drawable.BitmapDrawable(appResources, image);
            return {
                drawable: imageDrawable,
                height: image.getHeight()
            };
        }
        return new IconInfo();
    }
    getIconInfo(tabStripItem, color) {
        const originalIcon = this.getOriginalIcon(tabStripItem, color);
        return this.getDrawableInfo(originalIcon);
    }
    getFixedSizeIcon(image) {
        const inWidth = image.getWidth();
        const inHeight = image.getHeight();
        const iconSpecSize = getIconSpecSize({
            width: inWidth,
            height: inHeight
        });
        const widthPixels = iconSpecSize.width * Utils.layout.getDisplayDensity();
        const heightPixels = iconSpecSize.height * Utils.layout.getDisplayDensity();
        const scaledImage = android.graphics.Bitmap.createScaledBitmap(image, widthPixels, heightPixels, true);
        return scaledImage;
    }
    updateAndroidItemAt(index, spec) {
        // that try catch is fix for an android NPE error on css change which navigated in (not the current fragment)
        try {
            if (this.mTabsBar) {
                this.updateTabsBarItemAt(index, spec);
            }
        }
        catch (err) { }
    }
    getTabBarBackgroundColor() {
        return this.mTabsBar.getBackground();
    }
    setTabBarBackgroundColor(value) {
        if (value instanceof Color) {
            this.mTabsBar.setBackgroundColor(value.android);
        }
        else {
            this.mTabsBar.setBackground(tryCloneDrawable(value, this.nativeViewProtected.getResources()));
        }
    }
    getTabBarHighlightColor() {
        return getDefaultAccentColor(this._context);
    }
    setTabBarHighlightColor(value) {
        const color = value instanceof Color ? value.android : value;
        this.setTabsBarSelectedIndicatorColors([color]);
    }
    getTabBarSelectedItemColor() {
        return this.mSelectedItemColor;
    }
    setTabBarSelectedItemColor(value) {
        this.mSelectedItemColor = value;
    }
    getTabBarUnSelectedItemColor() {
        return this.mUnSelectedItemColor;
    }
    setTabBarUnSelectedItemColor(value) {
        this.mUnSelectedItemColor = value;
    }
    updateItem(tabStripItem) {
        // TODO: Should figure out a way to do it directly with the the nativeView
        const tabStripItemIndex = this.tabStrip.items.indexOf(tabStripItem);
        const tabItemSpec = this.createTabItemSpec(tabStripItem);
        this.updateAndroidItemAt(tabStripItemIndex, tabItemSpec);
    }
    setTabBarItemTitle(tabStripItem, value) {
        this.updateItem(tabStripItem);
    }
    setTabBarItemBackgroundColor(tabStripItem, value) {
        this.updateItem(tabStripItem);
    }
    _setItemColor(tabStripItem) {
        if (!tabStripItem.nativeViewProtected) {
            return;
        }
        const itemColor = tabStripItem.index === this.selectedIndex ? this.mSelectedItemColor : this.mUnSelectedItemColor || tabStripItem.style.color;
        // set label color
        if (itemColor) {
            tabStripItem.nativeViewProtected.setTextColor(itemColor.android || null);
        }
        // set icon color
        this.setIconColor(tabStripItem, itemColor);
    }
    setIconColor(tabStripItem, color) {
        if (!tabStripItem.nativeViewProtected) {
            return;
        }
        const tabBarItem = this.getTabBarItemView(tabStripItem.index);
        const drawableInfo = this.getIconInfo(tabStripItem, color);
        const imgView = tabBarItem.getChildAt(0);
        imgView.setImageDrawable(drawableInfo.drawable);
        imgView.setColorFilter(color?.android || null);
    }
    setTabBarItemColor(tabStripItem, value) {
        const itemColor = tabStripItem.index === this.selectedIndex ? this.mSelectedItemColor : this.mUnSelectedItemColor;
        if (itemColor) {
            // the itemColor is set through the selectedItemColor and unSelectedItemColor properties
            // so it does not respect the css color
            return;
        }
        const androidColor = value instanceof Color ? value.android : value;
        tabStripItem.nativeViewProtected.setTextColor(androidColor);
    }
    setTabBarIconColor(tabStripItem, value) {
        const itemColor = tabStripItem.index === this.selectedIndex ? this.mSelectedItemColor : this.mUnSelectedItemColor;
        if (itemColor) {
            // the itemColor is set through the selectedItemColor and unSelectedItemColor properties
            // so it does not respect the css color
            return;
        }
        this.setIconColor(tabStripItem);
    }
    setTabBarIconSource(tabStripItem, value) {
        this.updateItem(tabStripItem);
    }
    setTabBarItemFontInternal(tabStripItem, value) {
        if (value.fontSize) {
            tabStripItem.nativeViewProtected.setTextSize(value.fontSize);
        }
        tabStripItem.nativeViewProtected.setTypeface(value.getAndroidTypeface());
    }
    setTabBarItemTextTransform(tabStripItem, value) {
        const nestedLabel = tabStripItem.label;
        const title = getTransformedText(nestedLabel.text, value);
        tabStripItem.nativeViewProtected.setText(title);
    }
    setTabBarTextTransform(value) {
        const items = this.tabStrip && this.tabStrip.items;
        if (items) {
            items.forEach((tabStripItem) => {
                if (tabStripItem.label && tabStripItem.nativeViewProtected) {
                    const nestedLabel = tabStripItem.label;
                    const title = getTransformedText(nestedLabel.text, value);
                    tabStripItem.nativeViewProtected.setText(title);
                }
            });
        }
        this.mTextTransform = value;
    }
    onTabsBarSelectedPositionChange(position, prevPosition) {
        this.mViewPager.setCurrentItem(position, this.animationEnabled);
        const tabStripItems = this.tabStrip?.items;
        if (position >= 0 && tabStripItems && tabStripItems[position]) {
            tabStripItems[position]._emit(TabStripItem.selectEvent);
            this._setItemColor(tabStripItems[position]);
        }
        if (prevPosition >= 0 && tabStripItems && tabStripItems[prevPosition]) {
            tabStripItems[prevPosition]._emit(TabStripItem.unselectEvent);
            this._setItemColor(tabStripItems[prevPosition]);
        }
    }
    onTabsBarTap(position) {
        const tabStrip = this.tabStrip;
        const tabStripItems = tabStrip && tabStrip.items;
        if (position >= 0 && tabStripItems[position]) {
            tabStripItems[position]._emit(TabStripItem.tapEvent);
            tabStrip.notify({ eventName: TabStrip.itemTapEvent, object: tabStrip, index: position });
        }
        if (!this.items[position]) {
            return false;
        }
        return true;
    }
    [selectedIndexProperty.setNative](value) {
        const current = this.mViewPager.getCurrentItem();
        if (current !== value) {
            this.mViewPager.setCurrentItem(value, this.animationEnabled);
        }
    }
    [itemsProperty.getDefault]() {
        return null;
    }
    [itemsProperty.setNative](value) {
        this.setItems(value);
        selectedIndexProperty.coerce(this);
    }
    [tabStripProperty.getDefault]() {
        return null;
    }
    [tabStripProperty.setNative](value) {
        this.setTabStripItems(value?.items || null);
    }
    [swipeEnabledProperty.getDefault]() {
        return true;
    }
    [swipeEnabledProperty.setNative](value) {
        this.mViewPager.setUserInputEnabled(value);
    }
    [offscreenTabLimitProperty.getDefault]() {
        return this.mViewPager.getOffscreenPageLimit();
    }
    [offscreenTabLimitProperty.setNative](value) {
        this.mViewPager.setOffscreenPageLimit(value);
    }
}
function tryCloneDrawable(value, resources) {
    if (value) {
        const constantState = value.getConstantState();
        if (constantState) {
            return constantState.newDrawable(resources);
        }
    }
    return value;
}
//# sourceMappingURL=index.android.js.map