var TabStripItem_1;
/**
 * @module @nativescript-community/ui-material-core-tabs/tab-strip-item
 */
import { CSSType, Image, Label, PseudoClassHandler, View } from '@nativescript/core';
import { backgroundColorProperty, backgroundInternalProperty } from '@nativescript/core/ui/styling/style-properties';
import { textTransformProperty } from '@nativescript/core/ui/text-base';
let TabStripItem = TabStripItem_1 = class TabStripItem extends View {
    get index() {
        return this.mIndex;
    }
    set index(value) {
        this.mIndex = value;
    }
    get title() {
        if (this.isLoaded) {
            return this.label.text;
        }
        return this.mTitle;
    }
    set title(value) {
        this.mTitle = value;
        if (this.isLoaded) {
            this.label.text = value;
        }
    }
    get iconClass() {
        if (this.isLoaded) {
            return this.image.className;
        }
        return this.mIconClass;
    }
    set iconClass(value) {
        this.mIconClass = value;
        if (this.isLoaded) {
            this.image.className = value;
        }
    }
    get iconSource() {
        if (this.isLoaded) {
            return this.image.src;
        }
        return this.mIconSource;
    }
    set iconSource(value) {
        this.mIconSource = value;
        if (this.isLoaded) {
            this.image.src = value;
        }
    }
    onLoaded() {
        if (!this.image) {
            const image = new Image();
            image.src = this.iconSource;
            image.className = this.iconClass;
            this.image = image;
            this._addView(this.image);
        }
        if (!this.label) {
            const label = new Label();
            label.text = this.title;
            this.label = label;
            this._addView(this.label);
        }
        super.onLoaded();
        this.mLabelColorHandler =
            this.mLabelColorHandler ||
                ((args) => {
                    const parent = this.parent;
                    const tabStripParent = parent && parent.parent;
                    return tabStripParent && tabStripParent.setTabBarItemColor(this, args.value);
                });
        this.label.style.on('colorChange', this.mLabelColorHandler);
        this.mLabelFontHandler =
            this.mLabelFontHandler ||
                ((args) => {
                    const parent = this.parent;
                    const tabStripParent = parent && parent.parent;
                    return tabStripParent && tabStripParent.setTabBarItemFontInternal(this, args.value);
                });
        this.label.style.on('fontInternalChange', this.mLabelFontHandler);
        this.mLabelTextTransformHandler =
            this.mLabelTextTransformHandler ||
                ((args) => {
                    const parent = this.parent;
                    const tabStripParent = parent && parent.parent;
                    return tabStripParent && tabStripParent.setTabBarItemTextTransform(this, args.value);
                });
        this.label.style.on('textTransformChange', this.mLabelTextTransformHandler);
        this._labelTextHandler =
            this._labelTextHandler ||
                ((args) => {
                    const parent = this.parent;
                    const tabStripParent = parent && parent.parent;
                    return tabStripParent && tabStripParent.setTabBarItemTitle(this, args.value);
                });
        this.label.on('textChange', this._labelTextHandler);
        this.mImageColorHandler =
            this.mImageColorHandler ||
                ((args) => {
                    const parent = this.parent;
                    const tabStripParent = parent && parent.parent;
                    return tabStripParent && tabStripParent.setTabBarIconColor(this, args.value);
                });
        this.image.style.on('colorChange', this.mImageColorHandler);
        this.mImageFontHandler =
            this.mImageFontHandler ||
                ((args) => {
                    const parent = this.parent;
                    const tabStripParent = parent && parent.parent;
                    return tabStripParent && tabStripParent.setTabBarItemFontInternal(this, args.value);
                });
        this.image.style.on('fontInternalChange', this.mImageFontHandler);
        this.mImageSrcHandler =
            this.mImageSrcHandler ||
                ((args) => {
                    const parent = this.parent;
                    const tabStripParent = parent && parent.parent;
                    return tabStripParent && tabStripParent.setTabBarIconSource(this, args.value);
                });
        this.image.on('srcChange', this.mImageSrcHandler);
    }
    onUnloaded() {
        super.onUnloaded();
        this.label.style.off('colorChange', this.mLabelColorHandler);
        this.label.style.off('fontInternalChange', this.mLabelFontHandler);
        this.label.style.off('textTransformChange', this.mLabelTextTransformHandler);
        this.label.style.off('textChange', this._labelTextHandler);
        this.image.style.off('colorChange', this.mImageColorHandler);
        this.image.style.off('fontInternalChange', this.mImageFontHandler);
        this.image.style.off('srcChange', this.mImageSrcHandler);
    }
    eachChild(callback) {
        if (this.label) {
            callback(this.label);
        }
        if (this.image) {
            callback(this.image);
        }
    }
    _addChildFromBuilder(name, value) {
        if (value instanceof Image) {
            this.image = value;
            this.iconSource = value.src;
            this.iconClass = value.className;
            this._addView(value);
            // selectedIndexProperty.coerce(this);
        }
        if (value instanceof Label) {
            this.label = value;
            this.title = value.text;
            this._addView(value);
            // selectedIndexProperty.coerce(this);
        }
    }
    requestLayout() {
        // Default implementation for non View instances (like TabViewItem).
        const parent = this.parent;
        if (parent) {
            parent.requestLayout();
        }
    }
    _updateTabStateChangeHandler(subscribe) {
        if (subscribe) {
            this.mHighlightedHandler =
                this.mHighlightedHandler ||
                    (() => {
                        this._goToVisualState('highlighted');
                    });
            this.mNormalHandler =
                this.mNormalHandler ||
                    (() => {
                        this._goToVisualState('normal');
                    });
            this.on(TabStripItem_1.selectEvent, this.mHighlightedHandler);
            this.on(TabStripItem_1.unselectEvent, this.mNormalHandler);
            const parent = this.parent;
            const tabStripParent = parent && parent.parent;
            if (this.mIndex === tabStripParent.selectedIndex && !(global.isIOS && tabStripParent.cssType.toLowerCase() === 'tabs')) {
                // HACK: tabStripParent instanceof Tabs creates a circular dependency
                // HACK: tabStripParent.cssType === "Tabs" is a hacky workaround
                this._goToVisualState('highlighted');
            }
        }
        else {
            this.off(TabStripItem_1.selectEvent, this.mHighlightedHandler);
            this.off(TabStripItem_1.unselectEvent, this.mNormalHandler);
        }
    }
    [backgroundColorProperty.getDefault]() {
        const tabStripParent = this.parent?.parent;
        return tabStripParent?.getTabBarBackgroundColor();
    }
    [backgroundColorProperty.setNative](value) {
        const tabStripParent = this.parent?.parent;
        return tabStripParent?.setTabBarItemBackgroundColor(this, value);
    }
    [textTransformProperty.getDefault]() {
        const parent = this.parent;
        const tabStripParent = parent && parent.parent;
        return tabStripParent && tabStripParent.getTabBarItemTextTransform(this);
    }
    [textTransformProperty.setNative](value) {
        const parent = this.parent;
        const tabStripParent = parent && parent.parent;
        return tabStripParent && tabStripParent.setTabBarItemTextTransform(this, value);
    }
    [backgroundInternalProperty.getDefault]() {
        return null;
    }
    [backgroundInternalProperty.setNative](value) {
        // disable the background CSS properties
    }
};
TabStripItem.tapEvent = 'tap';
TabStripItem.selectEvent = 'select';
TabStripItem.unselectEvent = 'unselect';
__decorate([
    PseudoClassHandler('normal', 'highlighted', 'pressed', 'active')
], TabStripItem.prototype, "_updateTabStateChangeHandler", null);
TabStripItem = TabStripItem_1 = __decorate([
    CSSType('MDTabStripItem')
], TabStripItem);
export { TabStripItem };
//# sourceMappingURL=index.js.map