/**
 * @module @nativescript-community/ui-material-core-tabs/tab-strip
 */
import { rippleColorAlphaProperty, rippleColorProperty } from '@nativescript-community/ui-material-core/cssproperties';
import { CSSType, Color, CssProperty, Property, Style, View, booleanConverter } from '@nativescript/core';
import { backgroundColorProperty, backgroundInternalProperty, colorProperty, fontInternalProperty } from '@nativescript/core/ui/styling/style-properties';
import { textTransformProperty } from '@nativescript/core/ui/text-base';
import { TabStripItem } from '../tab-strip-item';
// Place this on top because the webpack ts-loader doesn't work when export
// is after reference
export const highlightColorProperty = new CssProperty({
    name: 'highlightColor',
    cssName: 'highlight-color',
    equalityComparer: Color.equals,
    valueConverter: (v) => new Color(v)
});
export const selectedItemColorProperty = new CssProperty({
    name: 'selectedItemColor',
    cssName: 'selected-item-color',
    equalityComparer: Color.equals,
    valueConverter: (v) => new Color(v)
});
export const unSelectedItemColorProperty = new CssProperty({
    name: 'unSelectedItemColor',
    cssName: 'un-selected-item-color',
    equalityComparer: Color.equals,
    valueConverter: (v) => new Color(v)
});
let TabStrip = class TabStrip extends View {
    eachChild(callback) {
        const items = this.items;
        if (items) {
            items.forEach((item, i) => {
                callback(item);
            });
        }
    }
    _addArrayFromBuilder(name, value) {
        if (name === 'items') {
            this.items = value;
        }
    }
    _addChildFromBuilder(name, value) {
        if (value instanceof TabStripItem) {
            if (!this.items) {
                this.items = new Array();
            }
            this.items.push(value);
            this._addView(value);
            // selectedIndexProperty.coerce(this);
        }
    }
    onItemsChanged(oldItems, newItems) {
        if (oldItems) {
            oldItems.forEach((item) => this._removeView(item));
        }
        if (newItems) {
            newItems.forEach((item) => {
                this._addView(item);
            });
        }
    }
    [backgroundColorProperty.getDefault]() {
        const parent = this.parent;
        return parent?.getTabBarBackgroundColor();
        // always default to at least a solid white background as fallback
        // building with Xcode 13 causes bgView with no background to be fully transparent unless a css background-color is set - this allows original default behavior to work as it always did
        // bgView.backgroundColor = new Color('#fff').ios;;
    }
    [backgroundColorProperty.setNative](value) {
        const parent = this.parent;
        return parent?.setTabBarBackgroundColor(value);
    }
    [backgroundInternalProperty.getDefault]() {
        return null;
    }
    [backgroundInternalProperty.setNative](value) {
        // disable the background CSS properties
    }
    [colorProperty.getDefault]() {
        const parent = this.parent;
        return parent && parent.getTabBarColor();
    }
    [colorProperty.setNative](value) {
        const parent = this.parent;
        return parent && parent.setTabBarColor(value);
    }
    [fontInternalProperty.getDefault]() {
        const parent = this.parent;
        return parent && parent.getTabBarFontInternal();
    }
    [fontInternalProperty.setNative](value) {
        const parent = this.parent;
        return parent && parent.setTabBarFontInternal(value);
    }
    [textTransformProperty.getDefault]() {
        const parent = this.parent;
        return parent && parent.getTabBarTextTransform();
    }
    [textTransformProperty.setNative](value) {
        const parent = this.parent;
        return parent && parent.setTabBarTextTransform(value);
    }
    [highlightColorProperty.getDefault]() {
        const parent = this.parent;
        return parent && parent.getTabBarHighlightColor();
    }
    [highlightColorProperty.setNative](value) {
        const parent = this.parent;
        return parent && parent.setTabBarHighlightColor(value);
    }
    [selectedItemColorProperty.getDefault]() {
        const parent = this.parent;
        return parent && parent.getTabBarSelectedItemColor();
    }
    [selectedItemColorProperty.setNative](value) {
        const parent = this.parent;
        return parent && parent.setTabBarSelectedItemColor(value);
    }
    [unSelectedItemColorProperty.getDefault]() {
        const parent = this.parent;
        return parent && parent.getTabBarUnSelectedItemColor();
    }
    [unSelectedItemColorProperty.setNative](value) {
        const parent = this.parent;
        return parent && parent.setTabBarUnSelectedItemColor(value);
    }
    [rippleColorProperty.getDefault]() {
        const parent = this.parent;
        return parent && parent.getTabBarRippleColor();
    }
    [rippleColorProperty.setNative](value) {
        const parent = this.parent;
        return parent && parent.setTabBarRippleColor(value, this.rippleColorAlpha);
    }
    [rippleColorAlphaProperty.setNative](value) {
        const parent = this.parent;
        return parent && parent.setTabBarRippleColor(this.rippleColor, value);
    }
};
TabStrip.itemTapEvent = 'itemTap';
TabStrip = __decorate([
    CSSType('MDTabStrip')
], TabStrip);
export { TabStrip };
const itemsProperty = new Property({
    name: 'items',
    valueChanged: (target, oldValue, newValue) => {
        target.onItemsChanged(oldValue, newValue);
    }
});
itemsProperty.register(TabStrip);
export const iosIconRenderingModeProperty = new Property({ name: 'iosIconRenderingMode', defaultValue: 'automatic' });
iosIconRenderingModeProperty.register(TabStrip);
export const isIconSizeFixedProperty = new Property({
    name: 'isIconSizeFixed',
    defaultValue: true,
    valueConverter: booleanConverter
});
isIconSizeFixedProperty.register(TabStrip);
highlightColorProperty.register(Style);
selectedItemColorProperty.register(Style);
unSelectedItemColorProperty.register(Style);
//# sourceMappingURL=index.js.map